#import "NRAppDelegate.h"

#import "NRFetchOptionsController.h"
#import "NRListViewController.h"
#import "todointerface.h"


#define _DID_INIT_DB_KEY			@"didInitDB"


@interface NRAppDelegate ()

/// Holds a pointer to the root view controller.
@property (nonatomic, retain) UINavigationController *navigationController;

- (NSString*)dbPath;

@end


@implementation NRAppDelegate

@synthesize navigationController = navigationController_;
@synthesize window = window_;

#pragma mark -
#pragma mark Life

- (BOOL)application:(UIApplication *)application
	didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
	// Is this our first time running the example? If so, generate stuff.
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	const BOOL didInit = [defaults boolForKey:_DID_INIT_DB_KEY];
	if (!initTodoInterface([self dbPath], !didInit)) {
		abort();
		return NO;
	}

	// Save success for next launch.
	[defaults setBool:YES forKey:_DID_INIT_DB_KEY];
	[defaults synchronize];

	[NRFetchOptionsController loadFetchParams];

	// Create UI hierarchy.
	self.window = [[[UIWindow alloc]
		initWithFrame:[[UIScreen mainScreen] bounds]] autorelease];

	self.navigationController = [[[UINavigationController alloc]
		initWithRootViewController:[[NRListViewController new] autorelease]]
		autorelease];
	[self.window addSubview:self.navigationController.view];

	SEL action = @selector(setRootViewController:);
	if ([self.window respondsToSelector:action])
		[self.window performSelector:action
			withObject:self.navigationController];

	self.window.backgroundColor = [UIColor whiteColor];
	[self.window makeKeyAndVisible];
	return YES;
}

- (void)dealloc
{
	[navigationController_ release];
	[window_ release];
	[super dealloc];
}

/// Try to free up nimrod resources and clean up properly.
- (void)applicationWillTerminate:(UIApplication *)application
{
	closeTodoInterface();
}

/// On multitasking devices, try to save our configuration.
- (void)applicationDidEnterBackground:(UIApplication *)application
{
	closeTodoInterface();
}

/// On multitasking devices, recover our database connection.
- (void)applicationWillEnterForeground:(UIApplication *)application
{
	if (!reopenTodoInterface([self dbPath]))
		abort();
}

#pragma mark -
#pragma mark Methods

/// Returns the SQLite database file path.
- (NSString*)dbPath
{
	// Find out the path to our sqlite database.
	NSArray *paths = NSSearchPathForDirectoriesInDomains(
		NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *documentsDirectory = [paths objectAtIndex:0];
	return [documentsDirectory
		stringByAppendingPathComponent:@"nimTodo-sqlite3.db"];
}

@end
