#import "NRFetchOptionsController.h"

#import "NRPagedParams.h"
#import "todointerface.h"


#define _FETCH_CFG_KEY				@"fetchParamsSerialization"


@interface NRFetchOptionsController ()
@property (nonatomic, retain) NRPagedParams *params;

+ (void)saveFetchParams;

@end


@implementation NRFetchOptionsController

@synthesize params = params_;

#pragma mark -
#pragma mark Life

- (id)init
{
	return [self initWithStyle:UITableViewStyleGrouped];
}

- (id)initWithStyle:(UITableViewStyle)style
{
	if (!(self = [super initWithStyle:style]))
		return nil;

	self.params = [NRPagedParams get];
	return self;
}

- (void)viewDidLoad
{
	[super viewDidLoad];
	self.view.frame = self.view.bounds;

	self.title = @"Filter options";

	UIBarButtonItem *button = [[[UIBarButtonItem alloc]
		initWithBarButtonSystemItem:UIBarButtonSystemItemDone
		target:self action:@selector(doneButtonTouched)] autorelease];
	self.navigationItem.rightBarButtonItem = button;

	button = [[[UIBarButtonItem alloc]
		initWithBarButtonSystemItem:UIBarButtonSystemItemCancel
		target:self action:@selector(cancelButtonTouched)] autorelease];
	self.navigationItem.leftBarButtonItem = button;
}

- (void)dealloc
{
	[params_ release];
	[super dealloc];
}

#pragma mark -
#pragma mark Methods

- (BOOL)shouldAutorotateToInterfaceOrientation:
	(UIInterfaceOrientation)interfaceOrientation
{
	return YES;
}

/// Called when the user accepts the current selection of options.
- (void)doneButtonTouched
{
	[self.params save];
	[NRFetchOptionsController saveFetchParams];
	[self dismissModalViewControllerAnimated:YES];
}

/// Well, ignore changes, just return.
- (void)cancelButtonTouched
{
	[self dismissModalViewControllerAnimated:YES];
}

/// Called during startup, loads previous fetch params from user preferences.
+ (void)loadFetchParams
{
	// Retrieve, if available, previous list filtering settings.
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	NSString *previousFetchParams = [defaults stringForKey:_FETCH_CFG_KEY];
	if (previousFetchParams.length)
		unmarshalFetchParams(previousFetchParams);
}

/** Saves to the preferences the fetch parameters as JSON.
 * This is useful for the application to recover its previous state if the user
 * switches away and the app is killed or the device doesn't support
 * multitasking and has to exit.
 */
+ (void)saveFetchParams
{
	char *data = marshalFetchParams();
	if (!data || !*data)
		return;

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	[defaults setObject:[NSString stringWithUTF8String:data]
		forKey:_FETCH_CFG_KEY];
	[defaults synchronize];
}

#pragma mark -
#pragma mark Table

/// Complies with the protocol, tells the table how many items it has.
- (NSInteger)tableView:(UITableView *)tableView
	numberOfRowsInSection:(NSInteger)section
{
	return 4;
}

/// Builds the cell object for a specific index.
- (UITableViewCell *)tableView:(UITableView *)tableView
	cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
	static NSString *cell_id = @"optionsCell";

	// Dequeue or create a new cell.
	UITableViewCell *cell = [tableView
		dequeueReusableCellWithIdentifier:cell_id];

	if (nil == cell) {
		cell = [[[UITableViewCell alloc]
			initWithStyle:UITableViewCellStyleDefault
			reuseIdentifier:cell_id] autorelease];
		UISwitch *s = [[UISwitch new] autorelease];
		cell.accessoryView = s;
		[s addTarget:self action:@selector(switchChanged:)
			forControlEvents:UIControlEventValueChanged];
	}

	cell.tag = indexPath.row;
	UISwitch *s = (id)cell.accessoryView;
	switch (indexPath.row) {
		case 0:
			cell.textLabel.text = @"Ascending date?";
			s.on = self.params.dateAscending;
			break;
		case 1:
			cell.textLabel.text = @"Ascending priority?";
			s.on = self.params.priorityAscending;
			break;
		case 2:
			cell.textLabel.text = @"Show checked?";
			s.on = self.params.showChecked;
			break;
		default:
			cell.textLabel.text = @"Show unchecked?";
			s.on = self.params.showUnchecked;
			break;
	}

	return cell;
}

- (void)switchChanged:(id)sender
{
	UISwitch *s = (UISwitch *)sender;
	UITableViewCell *cell = (UITableViewCell*)s.superview;
	switch (cell.tag) {
		case 0: self.params.dateAscending = s.on; break;
		case 1: self.params.priorityAscending = s.on; break;
		case 2: self.params.showChecked = s.on; break;
		default: self.params.showUnchecked = s.on; break;
	}
}

@end
