#import "NRTodoCell.h"

#import "NRTodo.h"

#import "ELHASO.h"


#define _BUTTON_SIZE			44


@interface NRTodoCell ()

/// The button which shows the display check and recognizes touches.
@property (nonatomic, retain) UIButton *checkButton;

@end


@implementation NRTodoCell

@synthesize checkButton = checkButton_;
@synthesize todo = todo_;

#pragma mark -
#pragma mark Life

/// Adds our customization here, updating the layout and adding some widgets.
- (id)initWithStyle:(UITableViewCellStyle)style
	reuseIdentifier:(NSString *)reuseIdentifier
{
	if (!(self = [super initWithStyle:style reuseIdentifier:reuseIdentifier]))
		return nil;

	// Creates the checkbox button.
	self.checkButton = [UIButton buttonWithType:UIButtonTypeCustom];
	self.checkButton.titleLabel.font = [UIFont boldSystemFontOfSize:30];
	self.checkButton.titleLabel.adjustsFontSizeToFitWidth = YES;
	[self.checkButton setTitle:@"☑" forState:UIControlStateSelected];
	[self.checkButton setTitle:@"☑" forState:UIControlStateHighlighted];
	[self.checkButton setTitle:@"☐" forState:UIControlStateNormal];
	[self.checkButton setTitleColor:[UIColor blackColor]
		forState:UIControlStateNormal];
	[self.checkButton addTarget:self action:@selector(didTouchCheck)
		forControlEvents:UIControlEventTouchDown];
	[self addSubview:self.checkButton];
	self.checkButton.frame = CGRectMake(0, 0, _BUTTON_SIZE, _BUTTON_SIZE);

	content_view_.opaque = NO;
	content_view_.backgroundColor = [UIColor clearColor];
	content_view_.contentMode = UIViewContentModeLeft;
	self.contentMode = UIViewContentModeLeft;
	return self;
}

- (void)dealloc
{
	[checkButton_ release];
	[todo_ release];
	[super dealloc];
}

#pragma mark -
#pragma mark Properties

/// Sets the todo and updates the contents of the cell.
- (void)setTodo:(NRTodo*)todo
{
	[todo retain];
	[todo_ release];
	todo_ = todo;

	self.checkButton.selected = todo.isDone;
}

#pragma mark -
#pragma mark Methods

/// Forces a redisplay of our custom cell, or it won't support rotation.
- (void)layoutSubviews
{
	[super layoutSubviews];
	[self setNeedsDisplay];
}

- (void)draw_content:(CGRect)cell_rect
{
	static NSDateFormatter *date_formatter = nil;
	if (nil == date_formatter) {
		date_formatter = [NSDateFormatter new];
		[date_formatter setTimeStyle:NSDateFormatterMediumStyle];
		[date_formatter setDateStyle:NSDateFormatterMediumStyle];
	}

	CGContextRef context = UIGraphicsGetCurrentContext();

	/*
	if (self.selected || self.highlighted)
		[[UIColor blueColor] set];
	else
		[[UIColor whiteColor] set];
	CGContextFillRect(context, cell_rect);
	*/

	const CGFloat w = cell_rect.size.width - 47;

	// Draw the todo text.
	[[UIColor blackColor] set];
	CGContextSetShadowWithColor(context, CGSizeMake(0, 1), 1,
		[UIColor whiteColor].CGColor);
	[self.todo.text drawInRect:CGRectMake(47, 2, w, 21)
		withFont:[UIFont boldSystemFontOfSize:17]
		lineBreakMode:UILineBreakModeTailTruncation];

	// Draw the date.
	[[UIColor lightGrayColor] set];
	CGContextSetShadowWithColor(context, CGSizeMake(0, 0), 0, nil);
	NSString *text = [date_formatter stringFromDate:self.todo.modificationDate];
	CGRect rect = CGRectMake(47, 22, w, 23);
	[text drawInRect:rect withFont:[UIFont systemFontOfSize:14]
		lineBreakMode:UILineBreakModeTailTruncation];

	// Draw the priority, to right align it we need to get its size first.
	CGContextSetShadowWithColor(context, CGSizeMake(0, 1), 1,
		[UIColor whiteColor].CGColor);
	[[UIColor blackColor] set];
	UIFont *font = [UIFont boldSystemFontOfSize:14];
	text = [NSString stringWithFormat:@"Priority: %d ", self.todo.priority];
	const CGSize size = [text sizeWithFont:font forWidth:w
		lineBreakMode:UILineBreakModeTailTruncation];
	rect.origin.x += rect.size.width - size.width;
	rect.size.width = size.width;
	[text drawInRect:rect withFont:font
		lineBreakMode:UILineBreakModeTailTruncation];
}

/** Handles touches on the cell button.
 * This will directly change the isDone status and save the object to the
 * database, in which case an invalidation of the list may be needed. This,
 * however, is left to observers watching the NRTodoModified notification.
 */
- (void)didTouchCheck
{
	DLOG(@"Quick check on %@", self.todo);
	self.todo.isDone = !self.todo.isDone;
	[self.todo save];
}

@end
