#import "NRTodoViewController.h"

#import "NRTodo.h"
#import "todointerface.h"

#import "ELHASO.h"


#define _PICKER_FADE				0.3


@interface NRTodoViewController ()
// Hold elements of the interface so we can access/update them.
@property (nonatomic, retain) UITextField *textField;
@property (nonatomic, retain) UISwitch *doneSwitch;
@property (nonatomic, retain) UIButton *priorityButton;
@property (nonatomic, retain) UILabel *modificationLabel;
@property (nonatomic, retain) UIButton *cancelButton;
@property (nonatomic, retain) UILabel *doneLabel;
@property (nonatomic, retain) UIButton *deleteButton;
@property (nonatomic, retain) UIPickerView *pickerView;
@property (nonatomic, retain) UIButton *pickerShieldButton;
@property (nonatomic, assign) BOOL isModifying;

- (void)refreshUI;

@end


@implementation NRTodoViewController

@synthesize cancelButton = cancelButton_;
@synthesize creationMode = creationMode_;
@synthesize deleteButton = deleteButton_;
@synthesize doneLabel = isDoneLabel_;
@synthesize doneSwitch = doneSwitch_;
@synthesize isModifying = isModifying_;
@synthesize modificationLabel = modificationLabel_;
@synthesize pickerShieldButton = pickerShieldButton_;
@synthesize pickerView = pickerView_;
@synthesize priorityButton = priorityButton_;
@synthesize textField = textField_;

#pragma mark -
#pragma mark Life

- (void)loadView
{
	[super loadView];
	self.view.autoresizingMask = UIViewAutoresizingFlexibleWidth |
		UIViewAutoresizingFlexibleHeight;
	self.view.frame = CGRectMake(0, 0, 320, 460);
	self.view.backgroundColor = [UIColor whiteColor];
	self.title = self.creationMode ? @"New todo" : @"";

	if (self.creationMode) {
		// Pepare nativagion buttons if this was presented for new addition.
		self.navigationItem.rightBarButtonItem = [[[UIBarButtonItem alloc]
			initWithBarButtonSystemItem:UIBarButtonSystemItemDone
			target:self action:@selector(saveTodo)] autorelease];
		self.navigationItem.rightBarButtonItem.enabled = NO;

		self.navigationItem.leftBarButtonItem = [[[UIBarButtonItem alloc]
			initWithBarButtonSystemItem:UIBarButtonSystemItemCancel
			target:self action:@selector(cancelTodo)] autorelease];
	} else {
		self.navigationItem.rightBarButtonItem = [[[UIBarButtonItem alloc]
			initWithBarButtonSystemItem:UIBarButtonSystemItemEdit
			target:self action:@selector(editTodo)] autorelease];
		self.navigationItem.rightBarButtonItem.enabled = YES;
	}

	// Create a background button to catch stray clicks, to dismiss keyboard.
	self.cancelButton = [UIButton buttonWithType:UIButtonTypeCustom];
	self.cancelButton.frame = self.view.bounds;
	self.cancelButton.autoresizingMask = UIViewAutoresizingFlexibleWidth |
		UIViewAutoresizingFlexibleHeight;
	[self.cancelButton addTarget:self action:@selector(didTouchOutside)
		forControlEvents:UIControlEventTouchDown];

	[self.view addSubview:self.cancelButton];

	self.textField = [[[UITextField alloc]
		initWithFrame:CGRectMake(20, 20, 280, 30)] autorelease];
	self.textField.borderStyle = UITextBorderStyleRoundedRect;
	self.textField.placeholder = @"Type here the todo";
	self.textField.delegate = self;
	self.textField.autoresizingMask = UIViewAutoresizingFlexibleWidth |
		UIViewAutoresizingFlexibleBottomMargin |
		UIViewAutoresizingFlexibleRightMargin;
	[self.view addSubview:self.textField];

	UILabel *label = [[[UILabel alloc]
		initWithFrame:CGRectMake(20, 58, 51, 21)] autorelease];
	label.text = @"Done?";
	[self.view addSubview:label];
	self.doneLabel = label;

	self.doneSwitch = [[[UISwitch alloc]
		initWithFrame:CGRectMake(20, 90, 79, 30)] autorelease];
	self.doneSwitch.enabled = NO;
	[self.doneSwitch addTarget:self action:@selector(didTouchSwitch)
		forControlEvents:UIControlEventValueChanged];

	[self.view addSubview:self.doneSwitch];

	label = [[[UILabel alloc] initWithFrame:CGRectMake(114, 65, 114, 43)]
		autorelease];
	label.text = @"Priority:";
	label.textAlignment = UITextAlignmentRight;
	[self.view addSubview:label];

	self.priorityButton = [UIButton buttonWithType:UIButtonTypeRoundedRect];
	self.priorityButton.frame = CGRectMake(236, 65, 64, 44);
	[self.priorityButton addTarget:self action:@selector(didTouchPriority)
		forControlEvents:UIControlEventTouchUpInside];
	[self.view addSubview:self.priorityButton];

	label = [[[UILabel alloc] initWithFrame:CGRectMake(20, 116, 280, 55)]
		autorelease];
	label.backgroundColor = [UIColor clearColor];
	label.text = @"Modification date";
	label.numberOfLines = 0;
	label.autoresizingMask = UIViewAutoresizingFlexibleWidth;
	[self.view addSubview:label];
	self.modificationLabel = label;

	// Delete button at the bottom.
	self.deleteButton = [UIButton buttonWithType:UIButtonTypeRoundedRect];
	self.deleteButton.frame = CGRectMake(20, 397, 280, 44);
	self.deleteButton.autoresizingMask = UIViewAutoresizingFlexibleWidth |
		UIViewAutoresizingFlexibleTopMargin;
	[self.deleteButton setTitle:@"Delete todo" forState:UIControlStateNormal];
	[self.deleteButton addTarget:self action:@selector(didTouchDeleteButton)
		forControlEvents:UIControlEventTouchUpInside];
	[self.view addSubview:self.deleteButton];
}

- (void)viewDidLoad
{
	[super viewDidLoad];

	if (self.creationMode)
		self.todo = [[NRTodo new] autorelease];

	[self refreshUI];
}

- (void)dealloc
{
	[pickerShieldButton_ release];
	[pickerView_ release];
	[deleteButton_ release];
	[cancelButton_ release];
	[isDoneLabel_ release];
	[modificationLabel_ release];
	[priorityButton_ release];
	[doneSwitch_ release];
	[textField_ release];
	[super dealloc];
}

#pragma mark -
#pragma mark Methods

- (BOOL)shouldAutorotateToInterfaceOrientation:
	(UIInterfaceOrientation)interfaceOrientation
{
	return YES;
}

/// Updates the user interface to match the current state of the logic.
- (void)refreshUI
{
	static NSDateFormatter *date_formatter = nil;
	if (nil == date_formatter) {
		date_formatter = [NSDateFormatter new];
		[date_formatter setTimeStyle:NSDateFormatterMediumStyle];
		[date_formatter setDateStyle:NSDateFormatterMediumStyle];
	}

	if (self.todo.modificationDate) {
		self.modificationLabel.text = [NSString
			stringWithFormat:@"Last modified:\n%@",
			[date_formatter stringFromDate:self.todo.modificationDate]];
	} else {
		self.modificationLabel.text = @"";
	}
	self.doneSwitch.on = self.todo.isDone;

	[self.priorityButton
		setTitle:[NSString stringWithFormat:@"%d", self.todo.priority]
		forState:UIControlStateNormal];
	self.textField.text = self.todo.text;

	self.modificationLabel.hidden = self.creationMode;
	self.doneLabel.hidden = self.creationMode;
	self.doneSwitch.hidden = self.creationMode;
	self.deleteButton.hidden = self.creationMode;

	if (!self.creationMode) {
		self.title = [NSString stringWithFormat:@"ID: %lld", self.todo.id_];
		self.doneSwitch.enabled = self.isModifying;
		self.priorityButton.enabled = self.isModifying;
		self.textField.enabled = self.isModifying;
	}
}

/// Called when the user touches the save button.
- (void)saveTodo
{
	DLOG(@"Saving todo %@", self.todo);
	if (addTodo(self.todo)) {
		DLOG(@"Got new todo %@", self.todo);
		[self dismissModalViewControllerAnimated:YES];
	} else {
		UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Error"
			message:@"Sorry, couldn't add todo to the database!"
			delegate:nil cancelButtonTitle:@"Bummer!"
			otherButtonTitles:nil];
		[alert show];
		[alert release];
	}
}

/// Called when the user touches the edit button.
- (void)editTodo
{
	DLOG(@"Entering edit mode for %@", self.todo);
	self.isModifying = YES;
	self.navigationItem.rightBarButtonItem = [[[UIBarButtonItem alloc]
		initWithBarButtonSystemItem:UIBarButtonSystemItemSave
		target:self action:@selector(modifyTodo)] autorelease];
	self.navigationItem.leftBarButtonItem = [[[UIBarButtonItem alloc]
		initWithBarButtonSystemItem:UIBarButtonSystemItemCancel
		target:self action:@selector(cancelEdit)] autorelease];

	[UIView beginAnimations:nil context:nil];
	self.deleteButton.enabled = NO;
	self.deleteButton.alpha = 0;
	[self refreshUI];
	[UIView commitAnimations];
}

/// The user changed her mind, discard changes and push back.
- (void)cancelTodo
{
	[self dismissModalViewControllerAnimated:YES];
}

/// The user changed her mind, discard changes and refresh UI.
- (void)cancelEdit
{
	self.navigationItem.leftBarButtonItem = nil;
	self.isModifying = NO;
	self.navigationItem.rightBarButtonItem = [[[UIBarButtonItem alloc]
		initWithBarButtonSystemItem:UIBarButtonSystemItemEdit
		target:self action:@selector(editTodo)] autorelease];
	[self.todo update];
	[self didCancelPicker];
	[self didTouchOutside];

	[UIView beginAnimations:nil context:nil];
	self.deleteButton.enabled = YES;
	self.deleteButton.alpha = 1;
	[self refreshUI];
	[UIView commitAnimations];
}

/// Save the current state for the user (piggybacks on cancelEdit).
- (void)modifyTodo
{
	[self.todo save];
	[self cancelEdit];
}

/// Consider outside touches for no button as cancellation of keyboard input.
- (void)didTouchOutside
{
	[self.textField resignFirstResponder];
	if (self.pickerView) {
		[self.pickerView removeFromSuperview];
		self.pickerView = nil;
	}
}

/// User touched the switch, update our logic value.
- (void)didTouchSwitch
{
	self.todo.isDone = self.doneSwitch.on;
	[self refreshUI];
}

/// The user pressed the priority, show a picker.
- (void)didTouchPriority
{
	[self.textField resignFirstResponder];

	// The first time allocate these items, later just reconfigure them.
	if (!self.pickerView) {
		self.pickerShieldButton = [UIButton buttonWithType:UIButtonTypeCustom];
		self.pickerView = [[[UIPickerView alloc] initWithFrame:CGRectZero]
			autorelease];
	}

	self.pickerShieldButton.frame = self.view.bounds;
	self.pickerShieldButton.backgroundColor = [[UIColor blackColor]
		colorWithAlphaComponent:0.4];
	self.pickerShieldButton.alpha = 0;
	self.pickerShieldButton.autoresizingMask = UIViewAutoresizingFlexibleWidth |
		UIViewAutoresizingFlexibleHeight;
	[self.pickerShieldButton addTarget:self action:@selector(didCancelPicker)
		forControlEvents:UIControlEventTouchDown];
	[self.view addSubview:self.pickerShieldButton];

	self.pickerView.autoresizingMask = UIViewAutoresizingFlexibleTopMargin |
		UIViewAutoresizingFlexibleWidth;
	self.pickerView.delegate = self;
	self.pickerView.dataSource = self;
	self.pickerView.showsSelectionIndicator = YES;
	CGRect rect = self.view.bounds;
	rect.origin.y = rect.size.height - 216;
	rect.size.height = 216;
	// Actually, displace height to perform slide in animation.
	rect.origin.y += rect.size.height;
	self.pickerView.frame = rect;
	[self.pickerShieldButton addSubview:self.pickerView];
	[self.pickerView selectRow:self.todo.priority inComponent:0 animated:NO];

	// Animate fade in.
	[UIView beginAnimations:nil context:nil];
	[UIView setAnimationDuration:_PICKER_FADE];
	self.pickerShieldButton.alpha = 1;
	rect.origin.y -= rect.size.height;
	self.pickerView.frame = rect;
	[UIView commitAnimations];
}

/// User pressed the black area, dismiss the picker "layer".
- (void)didCancelPicker
{
	CGRect rect = self.pickerView.frame;

	[UIView beginAnimations:nil context:nil];
	[UIView setAnimationDuration:_PICKER_FADE];
	self.pickerShieldButton.alpha = 0;
	rect.origin.y += rect.size.height;
	self.pickerView.frame = rect;
	[UIView commitAnimations];
}

/// The user killed the todo, push back the view.
- (void)didTouchDeleteButton
{
	if (deleteTodo(self.todo, true))
		[self.navigationController popViewControllerAnimated:YES];
}

#pragma mark -
#pragma mark UITextFieldDelegate protocol

/// If the user touches the return key, accept entry and remove button.
- (BOOL)textFieldShouldReturn:(UITextField *)textField
{
	[textField resignFirstResponder];
	return NO;
}

/// Monitor changes and turn on/off the done button.
- (BOOL)textField:(UITextField *)textField
	shouldChangeCharactersInRange:(NSRange)range
	replacementString:(NSString*)string
{
	self.todo.text = [[self.textField.text
		stringByReplacingCharactersInRange:range withString:string]
		stringByTrimmingCharactersInSet:
			[NSCharacterSet whitespaceAndNewlineCharacterSet]];
	self.navigationItem.rightBarButtonItem.enabled = self.todo.text.length > 0;
	return YES;
}

#pragma mark -
#pragma mark UIPickerViewDelegate protocol

- (void)pickerView:(UIPickerView *)pickerView
	didSelectRow:(NSInteger)row inComponent:(NSInteger)component
{
	self.todo.priority = row;
	[self refreshUI];
}

- (NSString *)pickerView:(UIPickerView *)pickerView
	titleForRow:(NSInteger)row forComponent:(NSInteger)component
{
	return [NSString stringWithFormat:@"%d", row];
}

// Return a centered label
- (UIView *)pickerView:(UIPickerView *)pickerView
	viewForRow:(NSInteger)row forComponent:(NSInteger)component
	reusingView:(UIView *)view
{
	UILabel *label = [[[UILabel alloc] initWithFrame:CGRectZero] autorelease];
	label.text = [self pickerView:pickerView
		titleForRow:row forComponent:component];
	label.textAlignment = UITextAlignmentCenter;
	label.font = [UIFont boldSystemFontOfSize:18];
	label.backgroundColor = [UIColor clearColor];
	return label;
}

#pragma mark -
#pragma mark UIPickerViewDataSource protocol

- (NSInteger)numberOfComponentsInPickerView:(UIPickerView *)pickerView
{
	return 1;
}

- (NSInteger)pickerView:(UIPickerView *)pickerView
	numberOfRowsInComponent:(NSInteger)component
{
	return 10;
}

@end
